@@ -136,6 +136,8 @@
 
 #define EVERMORE_CHANNELS	12
 
+#ifdef __UNUSED__
+
 /*@ +charint @*/
 gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
 			  size_t len)
@@ -167,11 +169,12 @@ gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
 	    cp++;
 	tp++;
     }
-    type = (unsigned char)getub(buf2, 2);
+    type = (unsigned char)getub(buf2, 0);
     /*@ +usedef @*/
 
     /*@ -usedef -compdef @*/
-    gpsd_report(LOG_RAW, "EverMore packet type 0x%02x\n", type);
+    gpsd_report(session->context->debug, LOG_RAW,
+		"EverMore packet type 0x%02x (%zd bytes)\n", type, tp-buf2);
     /*@ +usedef +compdef @*/
 
     (void)snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag),
@@ -209,7 +212,7 @@ gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
 			   "%3.2f", version);
 	    mask |= DEVICEID_SET;
 	}
-	gpsd_report(LOG_DATA,
+	gpsd_report(session->context->debug, LOG_DATA,
 		    "NDO 0x02: time=%.2f, lat=%.2f lon=%.2f alt=%.2f speed=%.2f track=%.2f climb=%.2f mode=%d subtype='%s\n",
 		    session->newdata.time, session->newdata.latitude,
 		    session->newdata.longitude, session->newdata.altitude,
@@ -254,7 +257,7 @@ gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
 	}
 	/* that's all the information in this packet */
 	mask = TIME_SET | PPSTIME_IS | DOP_SET | MODE_SET | STATUS_SET;
-	gpsd_report(LOG_DATA,
+	gpsd_report(session->context->debug, LOG_DATA,
 		    "DDO 0x04: gdop=%.2f pdop=%.2f hdop=%.2f vdop=%.2f tdop=%.2f mode=%d, status=%d mask={TIME| DOP|MODE|STATUS}\n",
 		    session->gpsdata.dop.gdop, session->gpsdata.dop.pdop,
 		    session->gpsdata.dop.hdop, session->gpsdata.dop.vdop,
@@ -270,7 +273,7 @@ gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
 	gpsd_zero_satellites(&session->gpsdata);
 	memset(session->gpsdata.used, 0, sizeof(session->gpsdata.used));
 	if (session->gpsdata.satellites_visible > 12) {
-	    gpsd_report(LOG_WARN,
+	    gpsd_report(session->context->debug, LOG_WARN,
 			"Warning: EverMore packet has information about %d satellites!\n",
 			session->gpsdata.satellites_visible);
 	}
@@ -309,7 +312,7 @@ gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
 	session->gpsdata.satellites_visible = (int)satcnt;
 	/* that's all the information in this packet */
 	mask = SATELLITE_SET | USED_IS;
-	gpsd_report(LOG_DATA,
+	gpsd_report(session->context->debug, LOG_DATA,
 		    "CSO 0x06: time=%.2f used=%d visible=%d mask={TIME|SATELLITE|USED}\n",
 		    session->newdata.time, session->gpsdata.satellites_used,
 		    session->gpsdata.satellites_visible);
@@ -352,24 +355,28 @@ gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
 	}
 #undef SBITS
 #undef UBITS
-	gpsd_report(LOG_DATA, "MDO 0x04: time=%.2f mask={TIME|RAW}\n",
+	gpsd_report(session->context->debug, LOG_DATA,
+		    "MDO 0x04: time=%.2f mask={TIME|RAW}\n",
 		    session->newdata.time);
 	return TIME_SET | PPSTIME_IS | RAW_IS;
 
     case 0x20:			/* LogConfig Info, could be used as a probe for EverMore GPS */
-	gpsd_report(LOG_IO, "LogConfig EverMore packet, length %zd\n", datalen);
+	gpsd_report(session->context->debug, LOG_DATA,
+		    "LogConfig EverMore packet, length %zd\n", datalen);
 	return ONLINE_SET;
 
     case 0x22:			/* LogData */
-	gpsd_report(LOG_IO, "LogData EverMore packet, length %zd\n", datalen);
+	gpsd_report(session->context->debug, LOG_DATA,
+		    "LogData EverMore packet, length %zd\n", datalen);
 	return ONLINE_SET;
 
     case 0x38:			/* ACK */
-	gpsd_report(LOG_PROG, "EverMore command %02X ACK\n", getub(buf2, 3));
+	gpsd_report(session->context->debug, LOG_PROG,
+		    "EverMore command %02X ACK\n", getub(buf2, 3));
 	return ONLINE_SET;
 
     default:
-	gpsd_report(LOG_WARN,
+	gpsd_report(session->context->debug, LOG_WARN,
 		    "unknown EverMore packet EID 0x%02x, length %zd\n",
 		    buf2[0], datalen);
 	return 0;
@@ -386,15 +393,19 @@ static gps_mask_t evermore_parse_input(struct gps_device_t *session)
 	st = evermore_parse(session, session->packet.outbuffer,
 			    session->packet.outbuflen);
 	return st;
+    }
 #ifdef NMEA_ENABLE
-    } else if (session->packet.type == NMEA_PACKET) {
+    else if (session->packet.type == NMEA_PACKET) {
 	st = nmea_parse((char *)session->packet.outbuffer, session);
 	return st;
+    }
 #endif /* NMEA_ENABLE */
-    } else
+    else
 	return 0;
 }
 
+#endif /* __UNUSED__ */
+
 /*@ +charint -usedef -compdef @*/
 static ssize_t evermore_control_send(struct gps_device_t *session, char *buf,
 				     size_t len)
@@ -452,7 +463,8 @@ static bool evermore_protocol(struct gps_device_t *session, int protocol)
 	(char)0x00,		/* 3: reserved */
     };
     /*@ -charint */
-    gpsd_report(LOG_PROG, "evermore_protocol(%d)\n", protocol);
+    gpsd_report(session->context->debug, LOG_PROG,
+		"evermore_protocol(%d)\n", protocol);
     /*@i1@*/ tmp8 = (protocol != 0) ? 1 : 0;
     /* NMEA : binary */
     evrm_protocol_config[1] = tmp8;
@@ -482,7 +494,8 @@ static bool evermore_nmea_config(struct gps_device_t *session, int mode)
 	0, 0, 0, 0, 0, 0,	/* 10-15: reserved */
     };
     /*@ -charint */
-    gpsd_report(LOG_PROG, "evermore_nmea_config(%d)\n", mode);
+    gpsd_report(session->context->debug, LOG_PROG,
+		"evermore_nmea_config(%d)\n", mode);
     /*@i1@*/ tmp8 = (mode == 1) ? 5 : 1;
     /* NMEA GPGSV, gpsd  */
     evrm_nmeaout_config[6] = tmp8;	/* GPGSV, 1s or 5s */
@@ -495,18 +508,17 @@ static bool evermore_nmea_config(struct gps_device_t *session, int mode)
 
 static void evermore_mode(struct gps_device_t *session, int mode)
 {
-    gpsd_report(LOG_PROG, "evermore_mode(%d), %d\n", mode,
+    gpsd_report(session->context->debug, LOG_PROG,
+		"evermore_mode(%d), %d\n", mode,
 		session->back_to_nmea ? 1 : 0);
     if (mode == MODE_NMEA) {
 	/* NMEA */
 	(void)evermore_protocol(session, 1);
-	session->gpsdata.dev.driver_mode = MODE_NMEA;
 	(void)evermore_nmea_config(session, 1);	/* configure NMEA messages for gpsd */
     } else {
 	/* binary */
 	(void)evermore_protocol(session, 0);
 	session->back_to_nmea = false;
-	session->gpsdata.dev.driver_mode = MODE_BINARY;
     }
 }
 
@@ -540,7 +552,8 @@ static bool evermore_speed(struct gps_device_t *session,
 			   speed_t speed, char parity, int stopbits)
 {
     /*@ -type @*/
-    gpsd_report(LOG_PROG, "evermore_speed(%u%c%d)\n", speed, parity,
+    gpsd_report(session->context->debug, LOG_PROG,
+		"evermore_speed(%u%c%d)\n", (unsigned int)speed, parity,
 		stopbits);
     /* parity and stopbit switching aren't available on this chip */
     if (parity != session->gpsdata.dev.parity
@@ -582,7 +595,7 @@ static bool evermore_rate_switcher(struct gps_device_t *session, double rate)
 {
     /*@ +charint @*/
     if (rate < 1 || rate > 10) {
-	gpsd_report(LOG_ERROR, "valid rate range is 1-10.\n");
+	gpsd_report(session->context->debug, LOG_ERROR, "valid rate range is 1-10.\n");
 	return false;
     } else {
 	unsigned char evrm_rate_config[] = {
@@ -602,16 +615,16 @@ static bool evermore_rate_switcher(struct gps_device_t *session, double rate)
 
 /* this is everything we export */
 /* *INDENT-OFF* */
-const struct gps_type_t evermore_binary =
+const struct gps_type_t driver_evermore =
 {
-    .type_name      = "EverMore binary",	/* full name of type */
+    .type_name      = "EverMore",		/* full name of type */
     .packet_type    = EVERMORE_PACKET,		/* lexer packet type */
-    .flags	    = DRIVER_NOFLAGS,		/* no flags set */
-    .trigger        = "$PEMT,", 		/* recognize the type */
+    .flags	    = DRIVER_STICKY,		/* remember this */
+    .trigger        = NULL, 			/* recognize the type */
     .channels       = EVERMORE_CHANNELS,	/* consumer-grade GPS */
     .probe_detect   = NULL,			/* no probe */
     .get_packet     = generic_get,		/* use generic one */
-    .parse_packet   = evermore_parse_input,	/* parse message packets */
+    .parse_packet   = generic_parse_input,	/* parse message packets */
     .rtcm_writer    = gpsd_write,		/* send RTCM data straight */
     .event_hook     = evermore_event_hook,	/* lifetime event handler */
 #ifdef RECONFIGURE_ENABLE
@@ -623,9 +636,9 @@ const struct gps_type_t evermore_binary =
 #ifdef CONTROLSEND_ENABLE
     .control_send   = evermore_control_send,	/* how to send a control string */
 #endif /* CONTROLSEND_ENABLE */
-#ifdef NTPSHM_ENABLE
-    .ntp_offset     = NULL,		/* no method for NTP fudge factor */
-#endif /* NTPSHM_ ENABLE */
+#ifdef TIMEHINT_ENABLE
+    .time_offset     = NULL,		/* no method for NTP fudge factor */
+#endif /* TIMEHINT_ENABLE */
 };
 /* *INDENT-ON* */
 #endif /* defined(EVERMORE_ENABLE) && defined(BINARY_ENABLE) */
