@@ -72,7 +72,8 @@ _proto__msg_navsol(struct gps_device_t *session, unsigned char *buf, size_t data
     if (data_len != _PROTO__NAVSOL_MSG_LEN)
 	return 0;
 
-    gpsd_report(LOG_IO, "_proto_ NAVSOL - navigation data\n");
+    gpsd_report(session->context->debug, LOG_DATA,
+		"_proto_ NAVSOL - navigation data\n");
     /* if this protocol has a way to test message validity, use it */
     flags = GET_FLAGS();
     if ((flags & _PROTO__SOLUTION_VALID) == 0)
@@ -120,7 +121,8 @@ _proto__msg_navsol(struct gps_device_t *session, unsigned char *buf, size_t data
      * the fields it potentially set and the transfer mask. Doing this
      * makes it relatively easy to track down data-management problems.
      */
-    gpsd_report(LOG_DATA, "NAVSOL: time=%.2f, lat=%.2f lon=%.2f alt=%.2f mode=%d status=%d\n",
+    gpsd_report(session->context->debug, LOG_DATA,
+		"NAVSOL: time=%.2f, lat=%.2f lon=%.2f alt=%.2f mode=%d status=%d\n",
 		session->newdata.time,
 		session->newdata.latitude,
 		session->newdata.longitude,
@@ -142,7 +144,8 @@ _proto__msg_utctime(struct gps_device_t *session, unsigned char *buf, size_t dat
     if (data_len != UTCTIME_MSG_LEN)
 	return 0;
 
-    gpsd_report(LOG_IO, "_proto_ UTCTIME - navigation data\n");
+    gpsd_report(session->context->debug, LOG_DATA,
+		"_proto_ UTCTIME - navigation data\n");
     /* if this protocol has a way to test message validity, use it */
     flags = GET_FLAGS();
     if ((flags & _PROTO__TIME_VALID) == 0)
@@ -168,7 +171,8 @@ _proto__msg_svinfo(struct gps_device_t *session, unsigned char *buf, size_t data
     if (data_len != SVINFO_MSG_LEN )
 	return 0;
 
-    gpsd_report(LOG_IO, "_proto_ SVINFO - navigation data\n");
+    gpsd_report(session->context->debug, LOG_DATA,
+		"_proto_ SVINFO - navigation data\n");
     /* if this protocol has a way to test message validity, use it */
     flags = GET_FLAGS();
     if ((flags & _PROTO__SVINFO_VALID) == 0)
@@ -183,7 +187,8 @@ _proto__msg_svinfo(struct gps_device_t *session, unsigned char *buf, size_t data
      */
     nchan = GET_NUMBER_OF_CHANNELS();
     if ((nchan < 1) || (nchan > MAXCHANNELS)) {
-	gpsd_report(LOG_INF, "too many channels reported\n");
+	gpsd_report(session->context->debug, LOG_INF,
+		    "too many channels reported\n");
 	return 0;
     }
     gpsd_zero_satellites(&session->gpsdata);
@@ -207,7 +212,7 @@ _proto__msg_svinfo(struct gps_device_t *session, unsigned char *buf, size_t data
     session->gpsdata.skyview_time = NaN;
     session->gpsdata.satellites_used = nsv;
     session->gpsdata.satellites_visible = st;
-    gpsd_report(LOG_DATA,
+    gpsd_report(session->context->debug, LOG_DATA,
 		"SVINFO: visible=%d used=%d mask={SATELLITE|USED}\n",
 		session->gpsdata.satellites_visible,
 		session->gpsdata.satellites_used);
@@ -226,7 +231,8 @@ _proto__msg_raw(struct gps_device_t *session, unsigned char *buf, size_t data_le
     if (data_len != RAW_MSG_LEN )
 	return 0;
 
-    gpsd_report(LOG_IO, "_proto_ RAW - raw measurements\n");
+    gpsd_report(session->context->debug, LOG_DATA,
+		"_proto_ RAW - raw measurements\n");
     /* if this protocol has a way to test message validity, use it */
     flags = GET_FLAGS();
     if ((flags & _PROTO__SVINFO_VALID) == 0)
@@ -241,7 +247,8 @@ _proto__msg_raw(struct gps_device_t *session, unsigned char *buf, size_t data_le
      */
     nchan = GET_NUMBER_OF_CHANNELS();
     if ((nchan < 1) || (nchan > MAXCHANNELS)) {
-	gpsd_report(LOG_INF, "too many channels reported\n");
+	gpsd_report(session->context->debug, LOG_INF,
+		    "too many channels reported\n");
 	return 0;
     }
 
@@ -285,7 +292,8 @@ gps_mask_t _proto__dispatch(struct gps_device_t *session, unsigned char *buf, si
     type = GET_MESSAGE_TYPE();
 
     /* we may need to dump the raw packet */
-    gpsd_report(LOG_RAW, "raw _proto_ packet type 0x%02x\n", type);
+    gpsd_report(session->context->debug, LOG_RAW,
+		"raw _proto_ packet type 0x%02x\n", type);
 
    /*
     * The tag field is only 8 bytes; be careful you do not overflow.
@@ -299,7 +307,8 @@ gps_mask_t _proto__dispatch(struct gps_device_t *session, unsigned char *buf, si
 	/* Deliver message to specific decoder based on message type */
 
     default:
-	gpsd_report(LOG_WARN, "unknown packet id %d length %d\n", type, len);
+	gpsd_report(session->context->debug, LOG_WARN,
+		    "unknown packet id %d length %d\n", type, len);
 	return 0;
     }
 }
@@ -349,7 +358,8 @@ static ssize_t _proto__control_send(struct gps_device_t *session,
 
    /* we may need to dump the message */
     return gpsd_write(session, session->msgbuf, session->msgbuflen);
-   gpsd_report(LOG_IO, "writing _proto_ control type %02x\n");
+   gpsd_report(session->context->debug, LOG_PROG,
+	       "writing _proto_ control type %02x\n");
    return gpsd_write(session, session->msgbuf, session->msgbuflen);
 }
 /*@ -charint +usedef +compdef @*/
@@ -419,17 +429,11 @@ static void _proto__event_hook(struct gps_device_t *session, event_t event)
  */
 static gps_mask_t _proto__parse_input(struct gps_device_t *session)
 {
-    gps_mask_t st;
-
     if (session->packet.type == _PROTO__PACKET) {
-	st = _proto__dispatch(session, session->packet.outbuffer, session->packet.outbuflen);
-	session->gpsdata.driver_mode = MODE_BINARY;
-	return st;
+	return _proto__dispatch(session, session->packet.outbuffer, session->packet.outbuflen);
 #ifdef NMEA_ENABLE
     } else if (session->packet.type == NMEA_PACKET) {
-	st = nmea_parse((char *)session->packet.outbuffer, session);
-	session->gpsdata.driver_mode = MODE_NMEA;
-	return st;
+	return nmea_parse((char *)session->packet.outbuffer, session);
 #endif /* NMEA_ENABLE */
     } else
 	return 0;
@@ -451,23 +455,16 @@ static bool _proto__set_speed(struct gps_device_t *session,
 static void _proto__set_mode(struct gps_device_t *session, int mode)
 {
     if (mode == MODE_NMEA) {
-	// _proto__to_nmea(session->gpsdata.gps_fd,session->gpsdata.baudrate); /* send the mode switch control string */
-	session->gpsdata.driver_mode = MODE_NMEA;
-	/*
-	 * Anticipatory switching works only when the packet getter is the
-	 * generic one and it recognizes packets of the type this driver
-	 * is expecting.  This should be the normal case.
-	 */
-	(void)gpsd_switch_driver(session, "Generic NMEA");
+	/* send a mode switch control string */
     } else {
+	/* send a mode switch control string */
 	session->back_to_nmea = false;
-	session->gpsdata.driver_mode = MODE_BINARY;
     }
 }
 #endif /* RECONFIGURE_ENABLE */
 
-#ifdef NTPSHM_ENABLE
-static double _proto_ntp_offset(struct gps_device_t *session)
+#ifdef TIMEHINT_ENABLE
+static double _proto_time_offset(struct gps_device_t *session)
 {
     /*
      * If NTP notification is enabled, the GPS will occasionally NTP
@@ -481,7 +478,7 @@ static double _proto_ntp_offset(struct gps_device_t *session)
      */
     return MAGIC_CONSTANT;
 }
-#endif /* NTPSHM_ENABLE */
+#endif /* TIMEHINT_ENABLE */
 
 static void _proto__wrapup(struct gps_device_t *session)
 {
@@ -503,9 +500,9 @@ static void _proto__wrapup(struct gps_device_t *session)
 
 /* This is everything we export */
 /* *INDENT-OFF* */
-const struct gps_type_t _proto__binary = {
+const struct gps_type_t driver__proto__binary = {
     /* Full name of type */
-    .type_name        = "_proto_ binary",
+    .type_name        = "_proto",
     /* Associated lexer packet type */
     .packet_type      = _PROTO__PACKET,
     /* Driver tyoe flags */
@@ -538,9 +535,9 @@ const struct gps_type_t _proto__binary = {
     /* Control string sender - should provide checksum and headers/trailer */
     .control_send   = _proto__control_send,
 #endif /* CONTROLSEND_ENABLE */
-#ifdef NTPSHM_ENABLE
-    .ntp_offset     = _proto_ntp_offset,
-#endif /* NTPSHM_ENABLE */
+#ifdef TIMEHINT_ENABLE
+    .time_offset     = _proto_time_offset,
+#endif /* TIMEHINT_ENABLE */
 /* *INDENT-ON* */
 };
 #endif /* defined(_PROTO__ENABLE) && defined(BINARY_ENABLE) */
