@@ -17,7 +17,7 @@ class json_error(exceptions.Exception):
         self.explanation = explanation
 
 class gpscommon:
-    "Isolate socket handling and buffering from the protcol interpretation."
+    "Isolate socket handling and buffering from the protocol interpretation."
     def __init__(self, host="127.0.0.1", port=GPSD_PORT, verbose=0):
         self.sock = None        # in case we blow up in connect
         self.linebuffer = ""
@@ -127,17 +127,14 @@ WATCH_JSON	= 0x000010	# JSON output
 WATCH_NMEA	= 0x000020	# output in NMEA
 WATCH_RARE	= 0x000040	# output of packets in hex
 WATCH_RAW	= 0x000080	# output of raw packets
-WATCH_SCALED	= 0x000100	# scale output to floats 
+WATCH_SCALED	= 0x000100	# scale output to floats
 WATCH_TIMING	= 0x000200	# timing information
 WATCH_SPLIT24	= 0x001000	# split AIS Type 24s
 WATCH_PPS	= 0x002000	# enable PPS in raw/NMEA
 WATCH_DEVICE	= 0x000800	# watch specific device
 
-class gpsjson(gpscommon):
+class gpsjson:
     "Basic JSON decoding."
-    def __init__(self):
-        gpscommon.__init__(self)
-
     def __iter__(self):
         return self
 
