@@ -53,7 +53,8 @@ POLICY_SET     	= (1<<29)
 LOGMESSAGE_SET	= (1<<30)
 ERROR_SET      	= (1<<31)
 TIMEDRIFT_SET	= (1<<32)
-SET_HIGH_BIT	= 33
+EOF_SET         = (1<<33)
+SET_HIGH_BIT	= 34
 UNION_SET      	= (RTCM2_SET|RTCM3_SET|SUBFRAME_SET|AIS_SET|VERSION_SET|DEVICELIST_SET|ERROR_SET|GST_SET)
 STATUS_NO_FIX = 0
 STATUS_FIX = 1
@@ -111,7 +112,7 @@ class gpsdata:
             )
 
     def __init__(self):
-        # Initialize all data members 
+        # Initialize all data members
         self.online = 0                 # NZ if GPS on, zero if not
 
         self.valid = 0
@@ -260,8 +261,8 @@ class gps(gpscommon, gpsdata, gpsjson):
             self.valid = ONLINE_SET | DEVICE_SET
             self.path        = self.data["path"]
             self.activated   = default("activated", None)
-            driver = default("driver", None, DEVICEID_SET) 
-            subtype = default("subtype", None, DEVICEID_SET) 
+            driver = default("driver", None, DEVICEID_SET)
+            subtype = default("subtype", None, DEVICEID_SET)
             self.gps_id      = driver
             if subtype:
                 self.gps_id += " " + subtype
@@ -297,7 +298,7 @@ class gps(gpscommon, gpsdata, gpsjson):
             for attrp in ("x", "y", "v", "h", "p", "g"):
                 setattr(self, attrp+"dop", default(attrp+"dop", NaN, DOP_SET))
             if "satellites" in self.data.keys():
-                self.satellites = [] 
+                self.satellites = []
                 for sat in self.data['satellites']:
                     self.satellites.append(gps.satellite(PRN=sat['PRN'], elevation=sat['el'], azimuth=sat['az'], ss=sat['ss'], used=sat['used']))
             self.satellites_used = 0
@@ -344,7 +345,7 @@ class gps(gpscommon, gpsdata, gpsjson):
         else: # flags & WATCH_ENABLE:
             if flags & WATCH_OLDSTYLE:
                 arg = 'w+'
-                if (flags & WATCH_NMEA):
+                if flags & WATCH_NMEA:
                     arg += 'r+'
                     return self.send(arg)
             else:
