@@ -14,6 +14,7 @@
 #include <signal.h>
 #include <time.h>
 #include <sys/time.h>
+#include <sys/select.h>
 #ifndef S_SPLINT_S
 #include <unistd.h>
 #endif /* S_SPLINT_S */
@@ -363,7 +364,7 @@ int main(int argc, char **argv)
 	if (gps_open(NULL, NULL, &gpsdata) != 0) {
 	    gpsd_report(context.debug, LOG_ERROR,
 			"no gpsd running or network error: %s.\n",
-			netlib_errstr(errno));
+			gps_errstr(errno));
 	    lowlevel = true;
 	}
     }
@@ -664,11 +665,19 @@ int main(int argc, char **argv)
 	    /* grab packets until we time out, get sync, or fail sync */
 	    for (hunting = true; hunting; )
 	    {
+#ifdef EFDS
+		fd_set efds;
+#endif /* EFDS */
 		switch(gpsd_await_data(&rfds, maxfd, &all_fds, context.debug))
 		{
 		case AWAIT_GOT_INPUT:
 		    break;
 		case AWAIT_NOT_READY:
+#ifdef EFDS
+		    /* no recovery from bad fd is possible */
+		    if (FD_ISSET(session.gpsdata.gps_fd, &efds))
+			exit(EXIT_FAILURE);
+#endif /* EFDS */
 		    continue;
 		case AWAIT_FAILED:
 		    exit(EXIT_FAILURE);
@@ -688,6 +697,10 @@ int main(int argc, char **argv)
 		    gpsd_report(context.debug, LOG_WARN,
 				"device error, bailing out.\n");
 		    exit(EXIT_FAILURE);
+		case DEVICE_EOF:
+		    gpsd_report(context.debug, LOG_WARN,
+				"device signed off, bailing out.\n");
+		    exit(EXIT_SUCCESS);
 		default:
 		    break;
 		}
