@@ -44,11 +44,6 @@
 
 #include "gpsd_config.h"
 
-#if defined(HAVE_LIBCAP) && !defined(S_SPLINT_S)
-#include <sys/capability.h>
-#include <sys/prctl.h>
-#endif /* HAVE_LIBCAP */
-
 #include "gpsd.h"
 #include "sockaddr.h"
 #include "gps_json.h"
@@ -154,6 +149,14 @@ static struct gps_context_t context;
 static int sd_socket_count = 0;
 #endif
 
+/* work around the unfinished ipv6 implementation on hurd */
+#ifdef __GNU__
+#ifndef IPV6_TCLASS
+#define IPV6_TCLASS 61
+#endif
+#endif
+
+
 static volatile sig_atomic_t signalled;
 
 static void onsig(int sig)
@@ -245,10 +248,13 @@ static void usage(void)
   -S integer (default %s) = set port for daemon \n\
   -h		     	    = help message \n\
   -V			    = emit version and exit.\n\
-A device may be a local serial device for GPS input, or a URL of the form:\n\
+A device may be a local serial device for GPS input, or a URL in one \n\
+of the following forms:\n\
+     tcp://host[:port]\n\
+     udp://host[:port]\n\
      {dgpsip|ntrip}://[user:passwd@]host[:port][/stream]\n\
      gpsd://host[:port][/device][?protocol]\n\
-in which case it specifies an input source for GPSD, DGPS or ntrip data.\n\
+in which case it specifies an input source for device, DGPS or ntrip data.\n\
 \n\
 The following driver types are compiled into this gpsd instance:\n",
 		 DEFAULT_GPSD_PORT);
@@ -335,6 +341,10 @@ static void adjust_max_fd(int fd, bool on)
 }
 
 #ifdef SOCKET_EXPORT_ENABLE
+#ifndef IPTOS_LOWDELAY
+#define IPTOS_LOWDELAY 0x10
+#endif
+
 static socket_t passivesock_af(int af, char *service, char *tcp_or_udp, int qlen)
 /* bind a passive command socket for the daemon */
 {
@@ -352,7 +362,7 @@ static socket_t passivesock_af(int af, char *service, char *tcp_or_udp, int qlen
     int type, proto, one = 1;
     in_port_t port;
     char *af_str = "";
-
+    const int dscp = IPTOS_LOWDELAY; /* Prioritize packet */
     INVALIDATE_SOCKET(s);
     if ((pse = getservbyname(service, tcp_or_udp)))
 	port = ntohs((in_port_t) pse->s_port);
@@ -391,6 +401,15 @@ static socket_t passivesock_af(int af, char *service, char *tcp_or_udp, int qlen
 	af_str = "IPv4";
 	/* see PF_INET6 case below */
 	s = socket(PF_INET, type, proto);
+	if (s > -1 ) {
+	/*@-unrecog@*/
+	/* Set packet priority */
+	if (setsockopt(s, IPPROTO_IP, IP_TOS, &dscp, sizeof(dscp)) == -1)
+	    gpsd_report(context.debug, LOG_WARN,
+			"Warning: SETSOCKOPT TOS failed\n");
+	}
+	/*@+unrecog@*/
+
 	break;
 #ifdef IPV6_ENABLE
     case AF_INET6:
@@ -432,6 +451,10 @@ static socket_t passivesock_af(int af, char *service, char *tcp_or_udp, int qlen
 		(void)close(s);
 		return -1;
 	    }
+	    /* Set packet priority */
+	    if (setsockopt(s, IPPROTO_IPV6, IPV6_TCLASS, &dscp, sizeof(dscp)) == -1)
+		gpsd_report(context.debug, LOG_WARN,
+			    "Warning: SETSOCKOPT TOS failed\n");
 	}
 #endif /* S_SPLINT_S */
 	break;
@@ -518,6 +541,7 @@ struct subscriber_t
     int fd;			/* client file descriptor. -1 if unused */
     timestamp_t active;		/* when subscriber last polled for data */
     struct policy_t policy;	/* configurable bits */
+    pthread_mutex_t mutex;	/* serialize access to fd */
 };
 
 #ifdef LIMITED_MAX_CLIENTS
@@ -533,6 +557,16 @@ static struct subscriber_t subscribers[MAXSUBSCRIBERS];	/* indexed by client fil
 
 #define UNALLOCATED_FD	-1
 
+static void lock_subscriber(struct subscriber_t *sub)
+{
+    (void)pthread_mutex_lock(&sub->mutex);
+}
+
+static void unlock_subscriber(struct subscriber_t *sub)
+{
+    (void)pthread_mutex_unlock(&sub->mutex);
+}
+
 static /*@null@*//*@observer@ */ struct subscriber_t *allocate_client(void)
 /* return the address of a subscriber structure allocated for a new session */
 {
@@ -554,8 +588,11 @@ static void detach_client(struct subscriber_t *sub)
 /* detach a client and terminate the session */
 {
     char *c_ip;
-    if (sub->fd == UNALLOCATED_FD)
+    lock_subscriber(sub);
+    if (sub->fd == UNALLOCATED_FD) {
+	unlock_subscriber(sub);
 	return;
+    }
     c_ip = netlib_sock2ip(sub->fd);
     (void)shutdown(sub->fd, SHUT_RDWR);
     gpsd_report(context.debug, LOG_SPIN,
@@ -577,6 +614,7 @@ static void detach_client(struct subscriber_t *sub)
     sub->policy.split24 = false;
     sub->policy.devpath[0] = '\0';
     sub->fd = UNALLOCATED_FD;
+    unlock_subscriber(sub);
     /*@+mustfreeonly@*/
 }
 
@@ -902,7 +940,7 @@ static void handle_control(int sfd, char *buf)
 		ignore_return(write(sfd, "ERROR\n", 6));
 	    }
 	}
-    } else if (strcmp(buf, "?devices")==0) {
+    } else if (strstr(buf, "?devices")==buf) {
 	/* write back devices list followed by OK */
 	for (devp = devices; devp < devices + MAXDEVICES; devp++) {
 	    char *path = devp->gpsdata.dev.path;
@@ -1524,38 +1562,16 @@ static void all_reports(struct gps_device_t *device, gps_mask_t changed)
 	//gpsd_report(context.debug, LOG_PROG, "NTP: No time this packet\n");
     } else if (isnan(device->newdata.time)) {
 	//gpsd_report(context.debug, LOG_PROG, "NTP: bad new time\n");
-    } else if (device->newdata.time == device->last_fixtime) {
+    } else if (device->newdata.time == device->last_fixtime.real) {
 	//gpsd_report(context.debug, LOG_PROG, "NTP: Not a new time\n");
     } else if (!device->ship_to_ntpd) {
 	//gpsd_report(context.debug, LOG_PROG, "NTP: No precision time report\n");
     } else {
-	double fix_time, integral, fractional;
-	struct timedrift_t td;
-
-#ifdef HAVE_CLOCK_GETTIME
-	/*@i2@*/(void)clock_gettime(CLOCK_REALTIME, &td.clock);
-#else
-	struct timeval clock_tv;
-	(void)gettimeofday(&clock_tv, NULL);
-	TVTOTS(&td.clock, &clock_tv);
-#endif /* HAVE_CLOCK_GETTIME */
-	fix_time = device->newdata.time;
-	/* assume zero when there's no offset method */
-	if (device->device_type == NULL
-	    || device->device_type->time_offset == NULL)
-	    fix_time += 0.0;
-	else
-	    fix_time += device->device_type->time_offset(device);
-	/* it's ugly but timestamp_t is double */
-	fractional = modf(fix_time, &integral);
-	/*@-type@*/ /* splint is confused about struct timespec */
-	td.real.tv_sec = (time_t)integral;
-	td.real.tv_nsec = (long)(fractional * 1e+9);
-	/*@+type@*/
 	/*@-compdef@*/
+	struct timedrift_t td;
+	ntpshm_latch(device, &td);
 	(void)ntpshm_put(device, device->shmIndex, &td);
 	/*@+compdef@*/
-	device->last_fixtime = device->newdata.time;
     }
 #endif /* NTPSHM_ENABLE */
 
@@ -1814,7 +1830,7 @@ int main(int argc, char *argv[])
     int i, option;
     int msocks[2] = {-1, -1};
     bool go_background = true;
-    bool in_restart;
+    volatile bool in_restart;
 
     context.debug = 0;
     gps_context_init(&context);
@@ -1879,7 +1895,7 @@ int main(int argc, char *argv[])
 
 #ifdef SYSTEMD_ENABLE
     sd_socket_count = sd_get_socket_count();
-    if (sd_socket_count > 0 && control_socket) {
+    if (sd_socket_count > 0 && control_socket != NULL) {
         gpsd_report(context.debug, LOG_WARN,
                     "control socket passed on command line ignored\n");
         control_socket = NULL;
@@ -2039,13 +2055,6 @@ int main(int argc, char *argv[])
 	struct passwd *pw;
 	struct stat stb;
 
-#if defined(HAVE_LIBCAP) && !defined(S_SPLINT_S)
- 	/* set flag: keep privileges across setuid() call */
-	if (prctl(PR_SET_KEEPCAPS, 1L, 0L, 0L, 0L) == -1)
-	    gpsd_report(context.debug, LOG_ERR,
-			"prctl(PR_SET_KEEPCAPS, 1L ) failed\n");
-#endif /* HAVE_LIBCAP */
-
 	/* make default devices accessible even after we drop privileges */
 	for (i = optind; i < argc; i++)
 	    /* coverity[toctou] */
@@ -2059,6 +2068,12 @@ int main(int argc, char *argv[])
 	 * of any compromises in the code.  It requires that all GPS
 	 * devices have their group read/write permissions set.
 	 */
+	/*@-nullpass@*/
+	if (setgroups(0, NULL) != 0)
+	    gpsd_report(context.debug, LOG_ERROR,
+			"setgroups() failed, errno %s\n",
+			strerror(errno));
+	/*@+nullpass@*/
 	/*@-type@*/
 #ifdef GPSD_GROUP
 	{
@@ -2087,21 +2102,6 @@ int main(int argc, char *argv[])
 			    "setuid() failed, errno %s\n",
 			    strerror(errno));
 	/*@+type@*/
-
- #if defined(HAVE_LIBCAP) && !defined(S_SPLINT_S)
-	/* drop root capabilities, except CAP_SYS_TIME for 1PPS support */
-	{
-	    cap_t caps = cap_from_text("cap_sys_time=pe");
-
-	    if (!caps)
-		gpsd_report(context.debug, LOG_ERR, "cap_from_text() failed.\n");
-	    else if (cap_set_proc(caps) == -1) {
-		gpsd_report(context.debug, LOG_ERR,
-			    "cap_set_proc() failed to drop root privs\n");
-		cap_free(caps);
-	    }
-	}
-#endif /* HAVE_LIBCAP */
     }
     gpsd_report(context.debug, LOG_INF,
 		"running with effective group ID %d\n", getegid());
@@ -2109,8 +2109,12 @@ int main(int argc, char *argv[])
 		"running with effective user ID %d\n", geteuid());
 
 #ifdef SOCKET_EXPORT_ENABLE
-    for (i = 0; i < NITEMS(subscribers); i++)
+    for (i = 0; i < NITEMS(subscribers); i++) {
 	subscribers[i].fd = UNALLOCATED_FD;
+#ifndef S_SPLINT_S
+	(void)pthread_mutex_init(&subscribers[i].mutex, NULL);
+#endif /* S_SPLINT_S */
+    }
 #endif /* SOCKET_EXPORT_ENABLE*/
 
     /*@-compdef -compdestroy@*/
@@ -2172,11 +2176,21 @@ int main(int argc, char *argv[])
 	}
 
     while (0 == signalled) {
+#ifdef EFDS
+	fd_set efds;
+#endif /* EFDS */
 	switch(gpsd_await_data(&rfds, maxfd, &all_fds, context.debug))
 	{
 	case AWAIT_GOT_INPUT:
 	    break;
 	case AWAIT_NOT_READY:
+#ifdef EFDS
+	    for (device = devices; device < devices + MAXDEVICES; device++)
+		if (FD_ISSET(device->gpsdata.gps_fd, &efds)) {
+		    deactivate_device(device);
+		    free_device(device);
+		}
+#endif /* EFDS*/
 	    continue;
 	case AWAIT_FAILED:
 	    exit(EXIT_FAILURE);
@@ -2297,6 +2311,7 @@ int main(int argc, char *argv[])
 		    adjust_max_fd(device->gpsdata.gps_fd, false);
 		    break;
 		case DEVICE_ERROR:
+		case DEVICE_EOF:
 		    deactivate_device(device);
 		    break;
 		default:
@@ -2323,10 +2338,13 @@ int main(int argc, char *argv[])
 	    if (sub->active == 0)
 		continue;
 
+	    lock_subscriber(sub);
 	    if (FD_ISSET(sub->fd, &rfds)) {
 		char buf[BUFSIZ];
 		int buflen;
 
+		unlock_subscriber(sub);
+
 		gpsd_report(context.debug, LOG_PROG,
 			    "checking client(%d)\n",
 			    sub_index(sub));
@@ -2351,6 +2369,8 @@ int main(int argc, char *argv[])
 			detach_client(sub);
 		}
 	    } else {
+		unlock_subscriber(sub);
+
 		if (!sub->policy.watcher
 		    && timestamp() - sub->active > COMMAND_TIMEOUT) {
 		    gpsd_report(context.debug, LOG_WARN,
