@@ -18,13 +18,16 @@
 #include <sys/time.h>		/* expected to declare select(2) a la SuS */
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/select.h>
 #include <fcntl.h>
 #ifndef S_SPLINT_S
 #include <unistd.h>
 #endif /* S_SPLINT_S */
 
 #include "gpsd.h"
+#include "gps_json.h"
 #include "gpsmon.h"
+#include "gpsdclient.h"
 #include "revision.h"
 
 #define BUFLEN		2048
@@ -35,7 +38,9 @@ extern struct monitor_object_t garmin_mmt, garmin_bin_ser_mmt;
 extern struct monitor_object_t italk_mmt, ubx_mmt, superstar2_mmt;
 extern struct monitor_object_t fv18_mmt, gpsclock_mmt, mtk3301_mmt;
 extern struct monitor_object_t oncore_mmt, tnt_mmt, aivdm_mmt;
+#ifdef NMEA_ENABLE
 extern const struct gps_type_t driver_nmea0183;
+#endif /* NMEA_ENABLE */
 
 /* These are public */
 struct gps_device_t session;
@@ -51,6 +56,7 @@ static WINDOW *statwin, *cmdwin;
 static char *type_name;
 static size_t promptlen = 0;
 struct termios cooked, rare;
+struct fixsource_t source;
 
 #ifdef PASSTHROUGH_ENABLE
 /* no methods, it's all device window */
@@ -337,8 +343,13 @@ static /*@observer@*/ const char *promptgen(void)
 		       9 - session.gpsdata.dev.stopbits,
 		       session.gpsdata.dev.parity,
 		       session.gpsdata.dev.stopbits);
-    else
+    else {
 	(void)strlcpy(buf, session.gpsdata.dev.path, sizeof(buf));
+	if (source.device != NULL) {
+	    (void) strlcat(buf, ":", sizeof(buf));
+	    (void) strlcat(buf, source.device, sizeof(buf));
+	}
+    }
     return buf;
 }
 
@@ -476,9 +487,11 @@ static void select_packet_monitor(struct gps_device_t *device)
      */
     if (device->packet.type != last_type) {
 	const struct gps_type_t *active_type = device->device_type;
+#ifdef NMEA_ENABLE
 	if (device->packet.type == NMEA_PACKET
 	    && ((device->device_type->flags & DRIVER_STICKY) != 0))
 	    active_type = &driver_nmea0183;
+#endif /* NMEA_ENABLE */
 	if (!switch_type(active_type))
 	    longjmp(terminate, TERM_DRIVER_SWITCH);
 	else {
@@ -631,7 +644,9 @@ ssize_t gpsd_write(struct gps_device_t *session,
 		   const size_t len)
 /* pass low-level data to devices, echoing it to the log window */
 {
+#if defined(CONTROLSEND_ENABLE) || defined(RECONFIGURE_ENABLE)
     monitor_dump_send((const char *)buf, len);
+#endif /* defined(CONTROLSEND_ENABLE) || defined(RECONFIGURE_ENABLE) */
     return gpsd_serial_write(session, buf, len);
 }
 
@@ -652,12 +667,8 @@ bool monitor_control_send( /*@in@*/ unsigned char *buf, size_t len)
 
 static bool monitor_raw_send( /*@in@*/ unsigned char *buf, size_t len)
 {
-    if (!serial)
-	return false;
-    else {
-	ssize_t st = gpsd_write(&session, (char *)buf, len);
-	return (st > 0 && (size_t) st == len);
-    }
+    ssize_t st = gpsd_write(&session, (char *)buf, len);
+    return (st > 0 && (size_t) st == len);
 }
 #endif /* CONTROLSEND_ENABLE */
 
@@ -687,6 +698,7 @@ static void gpsmon_hook(struct gps_device_t *device, gps_mask_t changed UNUSED)
 /* per-packet hook */
 {
     char buf[BUFSIZ];
+    struct timedrift_t td;
 
 #ifdef PPS_ENABLE
     if (!serial && strncmp((char*)device->packet.outbuffer, "{\"class\":\"PPS\",", 13) == 0)
@@ -723,6 +735,17 @@ static void gpsmon_hook(struct gps_device_t *device, gps_mask_t changed UNUSED)
     else
 #endif /* PPS_ENABLE */
     {
+#ifdef __future__
+	if (!serial)
+	{
+	    if (device->packet.type == JSON_PACKET)
+	    {
+		const char *end = NULL;
+		libgps_json_unpack((char *)device->packet.outbuffer, &session.gpsdata, &end);
+	    }
+	}
+#endif /* __future__ */
+
 	if (curses_active)
 	    select_packet_monitor(device);
 
@@ -756,7 +779,7 @@ static void gpsmon_hook(struct gps_device_t *device, gps_mask_t changed UNUSED)
     report_unlock();
 
     /* Update the last fix time seen for PPS. FIXME: do this here? */
-    device->last_fixtime = device->newdata.time;
+    ntpshm_latch(device, &td);
 }
 /*@+observertrans +nullpass +globstate +compdef +uniondef@*/
 
@@ -815,7 +838,9 @@ static bool do_command(const char *line)
 		context.readonly = !context.readonly;
 	    else
 		context.readonly = (atoi(line + 1) == 0);
+#ifdef RECONFIGURE_ENABLE
 	    announce_log("[probing %sabled]", context.readonly ? "dis" : "en");
+#endif /* RECONFIGURE_ENABLE */
 	    if (!context.readonly)
 		/* magic - forces a reconfigure */
 		session.packet.counter = 0;
@@ -1053,7 +1078,7 @@ static const char *cmdline;
 int main(int argc, char **argv)
 {
     int option;
-    char *explanation, *devicename;
+    char *explanation;
     int bailout = 0, matches = 0;
     bool nmea = false;
     fd_set all_fds;
@@ -1163,19 +1188,32 @@ int main(int argc, char **argv)
     gpsd_time_init(&context, time(NULL));
     gpsd_init(&session, &context, NULL);
 
-    if (optind >= argc)
-	devicename = "gpsd://localhost:" DEFAULT_GPSD_PORT;
-    else 
-	devicename = argv[optind];
+    /* Grok the server, port, and device. */
+    if (optind < argc) {
+	serial = (strncmp(argv[optind], "/dev", 4) == 0);
+	gpsd_source_spec(argv[optind], &source);
+    } else {
+	serial = false;
+	gpsd_source_spec(NULL, &source);
+    }
 
-    /* backward compatibilty: accept a bare server name */
-    if (strchr(devicename, ':') == NULL && devicename[0] != '/')
+    if (serial) {
+	assert(source.device != NULL);	/* clue to splint */
 	(void) strlcpy(session.gpsdata.dev.path, 
-		       "tcp://", sizeof(session.gpsdata.dev.path));
-    else
-	session.gpsdata.dev.path[0] = '\0';
-    (void)strlcat(session.gpsdata.dev.path, devicename,
-		  sizeof(session.gpsdata.dev.path));
+		       source.device, 
+		       sizeof(session.gpsdata.dev.path));
+    } else {
+	assert(source.server != NULL);	/* clue to splint */
+	if (strstr(source.server, "//") == 0)
+	    (void) strlcpy(session.gpsdata.dev.path, 
+			   "tcp://",
+			   sizeof(session.gpsdata.dev.path));
+	else
+	    session.gpsdata.dev.path[0] = '\0';
+	(void)snprintf(session.gpsdata.dev.path + strlen(session.gpsdata.dev.path),
+		       sizeof(session.gpsdata.dev.path) - strlen(session.gpsdata.dev.path),
+		       "%s:%s", source.server, source.port);
+    }
 
     if (gpsd_activate(&session, O_PROBEONLY) == -1) {
 	(void)fprintf(stderr,
@@ -1184,7 +1222,6 @@ int main(int argc, char **argv)
 	exit(EXIT_FAILURE);
     }
 
-    serial = (strncmp(devicename, "/dev", 4) == 0);
 
     if (serial) {
 #ifdef PPS_ENABLE
@@ -1193,8 +1230,10 @@ int main(int argc, char **argv)
 #endif /* PPS_ENABLE */
     }
     else {
-	/* FIXME: Also use WATCH*DEVICE here someday */
-	(void)gps_send(&session.gpsdata, nmea ? WATCHNMEA : WATCHRAW);
+	if (source.device != NULL)
+	    (void)gps_send(&session.gpsdata, nmea ? WATCHNMEADEVICE : WATCHRAWDEVICE, source.device);
+	else
+	    (void)gps_send(&session.gpsdata, nmea ? WATCHNMEA : WATCHRAW);
     }
 
     /*
@@ -1242,11 +1281,19 @@ int main(int argc, char **argv)
 
 	for (;;) 
 	{
+#ifdef EFDS
+	    fd_set efds;
+#endif /* EFDS */
 	    switch(gpsd_await_data(&rfds, maxfd, &all_fds, context.debug))
 	    {
 	    case AWAIT_GOT_INPUT:
 		break;
 	    case AWAIT_NOT_READY:
+#ifdef EFDS
+		/* no recovery from bad fd is possible */
+		if (FD_ISSET(session.gpsdata.gps_fd, &efds))
+		    longjmp(terminate, TERM_SELECT_FAILED);
+#endif /* EFDS */
 		continue;
 	    case AWAIT_FAILED:
 		longjmp(terminate, TERM_SELECT_FAILED);
@@ -1265,6 +1312,9 @@ int main(int argc, char **argv)
 	    case DEVICE_ERROR:
 		longjmp(terminate, TERM_READ_ERROR);
 		break;
+	    case DEVICE_EOF:
+		longjmp(terminate, TERM_QUIT);
+		break;
 	    default:
 		break;
 	    }
