@@ -23,8 +23,6 @@
  *
  */
 
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdbool.h>
@@ -35,6 +33,9 @@
 #include <termios.h>
 #include <time.h>
 #include <sys/time.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/select.h>
 #ifndef S_SPLINT_S
 #include <unistd.h>
 #endif /* S_SPLINT_S */
@@ -283,7 +284,7 @@ int main(int argc, char **argv)
     if (gps_open(source.server, source.port, &gpsdata) != 0) {
 	(void)fprintf(stderr,
 		      "gpspipe: could not connect to gpsd %s:%s, %s(%d)\n",
-		      source.server, source.port, strerror(errno), errno);
+		      source.server, source.port, gps_errstr(errno), errno);
 	exit(EXIT_FAILURE);
     }
     /*@ +nullpass +onlytrans @*/
