@@ -33,28 +33,26 @@
 
 #define CLIMB 3
 
-#include <sys/types.h>
-#include <sys/stat.h>
 #ifndef S_SPLINT_S
 #include <netdb.h>
 #ifndef AF_UNSPEC
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/socket.h>
 #endif /* AF_UNSPEC */
 #endif /* S_SPLINT_S */
 #ifndef INADDR_ANY
 #include <netinet/in.h>
 #endif /* INADDR_ANY */
+#ifndef S_SPLINT_S
+#include <arpa/inet.h>
+#include <unistd.h>
+#endif /* S_SPLINT_S */
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
 #include <errno.h>
 #include <stdio.h>
-#ifndef S_SPLINT_S
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <sys/socket.h>
-#include <unistd.h>
-#endif /* S_SPLINT_S */
 
 #include "gps.h"
 #include "gpsdclient.h"
@@ -78,13 +76,13 @@ int sd;
 
 /*  Read a line from a socket  */
 ssize_t sockreadline(int sockd,void *vptr,size_t maxlen) {
-  ssize_t n,rc;
+  ssize_t n;
   char    c,*buffer;
 
   buffer=vptr;
 
   for (n = 1; n < (ssize_t)maxlen; n++) {
-
+    ssize_t rc;
     if ((rc=read(sockd,&c,1))==1) {
       *buffer++=c;
       if (c=='\n')
@@ -110,13 +108,13 @@ ssize_t sockreadline(int sockd,void *vptr,size_t maxlen) {
 /*  Write a line to a socket  */
 ssize_t sockwriteline(int sockd,const void *vptr,size_t n) {
   size_t      nleft;
-  ssize_t     nwritten;
   const char *buffer;
 
   buffer=vptr;
   nleft=n;
 
   while (nleft>0) {
+    ssize_t     nwritten;
     if ((nwritten= write(sockd,buffer,nleft))<=0) {
       if (errno==EINTR)
         nwritten=0;
