@@ -19,10 +19,11 @@
 #include <stdarg.h>
 #include <fcntl.h>
 #include <ctype.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <syslog.h>
 #include <errno.h>
+#include <sys/types.h>
+#include <sys/select.h>
+#include <sys/stat.h>
 #ifndef S_SPLINT_S
 #include <sys/wait.h>
 #include <sys/socket.h>
@@ -903,6 +904,9 @@ static void gpsd_error_model(struct gps_device_t *session,
      * expected time error should be half the resolution of
      * the GPS clock, so we put the bound of the error
      * in as a constant pending getting it from each driver.
+     * FIXME: increase this if no keap-second has been seen
+     * and it's less than 750s (one almanac load cycle)
+     * from device powerup.
      */
     if (isnan(fix->time) == 0 && isnan(fix->ept) != 0)
 	fix->ept = 0.005;
@@ -943,7 +947,7 @@ static void gpsd_error_model(struct gps_device_t *session,
 	}
 	if ((fix->mode >= MODE_3D)
 	    && isnan(fix->epc) != 0 && fix->time > oldfix->time) {
-	    if (oldfix->mode > MODE_3D && fix->mode > MODE_3D) {
+	    if (oldfix->mode >= MODE_3D && fix->mode >= MODE_3D) {
 		timestamp_t t = fix->time - oldfix->time;
 		double e = oldfix->epv + fix->epv;
 		/* if vertical uncertainties are zero this will be too */
@@ -999,6 +1003,9 @@ int gpsd_await_data(/*@out@*/fd_set *rfds,
     struct timeval tv;
 #endif /* COMPAT_SELECT */
 
+#ifdef EFDS
+    FD_ZERO(efds);
+#endif /* EFDS */
     (void)memcpy((char *)rfds, (char *)all_fds, sizeof(fd_set));
     gpsd_report(debug, LOG_RAW + 2, "select waits\n");
     /*
@@ -1026,8 +1033,25 @@ int gpsd_await_data(/*@out@*/fd_set *rfds,
     if (status == -1) {
 	if (errno == EINTR)
 	    return AWAIT_NOT_READY;
-	gpsd_report(debug, LOG_ERROR, "select: %s\n", strerror(errno));
-	return AWAIT_FAILED;
+	else if (errno == EBADF) {
+	    int fd;
+	    for (fd = 0; fd < FD_SETSIZE; fd++)
+		/*
+		 * All we care about here is a cheap, fast, uninterruptible
+		 * way to check if a file descriptor is valid.
+		 * FIXME: pass out error fds when we can do a library bump.
+		 */
+		if (FD_ISSET(fd, all_fds) && fcntl(fd, F_GETFL, 0) == -1) {
+		    FD_CLR(fd, all_fds);
+#ifdef EFDS
+		    FD_SET(fd, efds);
+#endif /* EFDS */
+		}
+	    return AWAIT_NOT_READY;
+	} else {
+	    gpsd_report(debug, LOG_ERROR, "select: %s\n", strerror(errno));
+	    return AWAIT_FAILED;
+	}
     }
     /*@ +usedef +nullpass @*/
 
@@ -1143,9 +1167,9 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 	timestamp_t now = timestamp();
 	if (session->device_type != NULL && session->packet.start_time > 0) {
 #ifdef RECONFIGURE_ENABLE
-	    const time_t min_cycle = session->device_type->min_cycle;
+	    const double min_cycle = session->device_type->min_cycle;
 #else
-	    const time_t min_cycle = 1;
+	    const double min_cycle = 1;
 #endif /* RECONFIGURE_ENABLE */
 	    double quiet_time = (MINIMUM_QUIET_TIME * min_cycle);
 	    double gap = now - session->packet.start_time;
@@ -1211,8 +1235,14 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 		    "packet sniff on %s finds type %d\n",
 		    session->gpsdata.dev.path, session->packet.type);
 	if (session->packet.type == COMMENT_PACKET) {
-	    gpsd_report(session->context->debug, LOG_PROG,
-			"comment, sync lock deferred\n");
+	    if (strcmp((const char *)session->packet.outbuffer, "# EOF\n") == 0) {
+		gpsd_report(session->context->debug, LOG_PROG,
+			    "synthetic EOF\n");
+		return EOF_SET;
+	    }
+	    else
+		gpsd_report(session->context->debug, LOG_PROG,
+			    "comment, sync lock deferred\n");
 	    /* FALL THROUGH */
 	} else if (session->packet.type > COMMENT_PACKET) {
 	    if (session->device_type == NULL)
@@ -1232,8 +1262,12 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 		 * previous mode switch to binary succeeded in suppressing
 		 * NMEA).
 		 */
+#ifdef RECONFIGURE_ENABLE
 		bool dependent_nmea = (newtype == NMEA_PACKET
 				       && session->device_type->mode_switcher!=NULL);
+#else
+		bool dependent_nmea = false;
+#endif /* RECONFIGURE_ENABLE */
 
 		/*
 		 * Compute whether to switch drivers.
@@ -1469,7 +1503,12 @@ int gpsd_multipoll(const bool data_ready,
 	for (fragments = 0; ; fragments++) {
 	    gps_mask_t changed = gpsd_poll(device);
 
-	    if (changed == ERROR_SET) {
+	    if (changed == EOF_SET) {
+		gpsd_report(device->context->debug, LOG_WARN,
+			    "device signed off %s\n",
+			    device->gpsdata.dev.path);
+		return DEVICE_EOF;
+	    } else if (changed == ERROR_SET) {
 		gpsd_report(device->context->debug, LOG_WARN,
 			    "device read of %s returned error or packet sniffer failed sync (flags %s)\n",
 			    device->gpsdata.dev.path,
@@ -1602,3 +1641,36 @@ void gpsd_zero_satellites( /*@out@*/ struct gps_data_t *out)
     gps_clear_dop(&out->dop);
 #endif
 }
+
+void ntpshm_latch(struct gps_device_t *device, struct timedrift_t /*@out@*/*td)
+/* latch the fact that we've saved a fix */
+{
+    double fix_time, integral, fractional;
+
+#ifdef HAVE_CLOCK_GETTIME
+    /*@i2@*/(void)clock_gettime(CLOCK_REALTIME, &td->clock);
+#else
+    struct timeval clock_tv;
+    (void)gettimeofday(&clock_tv, NULL);
+    TVTOTS(&td->clock, &clock_tv);
+#endif /* HAVE_CLOCK_GETTIME */
+    fix_time = device->newdata.time;
+    /* assume zero when there's no offset method */
+    if (device->device_type == NULL
+	|| device->device_type->time_offset == NULL)
+	fix_time += 0.0;
+    else
+	fix_time += device->device_type->time_offset(device);
+    /* it's ugly but timestamp_t is double */
+    fractional = modf(fix_time, &integral);
+    /*@-type@*/ /* splint is confused about struct timespec */
+    td->real.tv_sec = (time_t)integral;
+    td->real.tv_nsec = (long)(fractional * 1e+9);
+    /*@+type@*/
+    device->last_fixtime.real = device->newdata.time;
+#ifndef S_SPLINT_S
+    device->last_fixtime.clock = td->clock.tv_sec + td->clock.tv_nsec / 1e9;
+#endif /* S_SPLINT_S */
+}
+
+/* end */
