@@ -151,7 +151,7 @@ static bool oncore_initialize(void)
 	(void)wattrset(Aywin, A_BOLD);
     (void)mvwprintw(Aywin, 1, 1, "PPS offset:");
 #ifndef PPS_ENABLE
-    (void)mvwaddstr(Aywin, 1, 13, "Not available");
+    (void)mvwaddstr(Aywin, 1, 13, "N/A");
 #endif /* PPS_ENABLE */
     (void)mvwprintw(Aywin, 3, 4, " @@Ay ");
     (void)wattrset(Aywin, A_NORMAL);
