@@ -276,6 +276,115 @@ static void gpsd_binary_almanac_dump(struct gps_device_t *session,
     }
 }
 
+#ifdef AIVDM_ENABLE
+
+#define GETLEFT(a) (((a%6) == 0) ? 0 : (6 - (a%6))) 
+
+static void gpsd_binary_ais_dump(struct gps_device_t *session,
+				     char bufp[], size_t len)
+{
+    char type[8] = "!AIVDM";
+    unsigned char data[256];
+    unsigned int msg1, msg2;
+    char numc[4];
+    char channel;
+    unsigned int left;
+    unsigned int datalen;
+    unsigned int offset;
+  
+    channel = 'A';
+    if (session->driver.aivdm.ais_channel == 'B') {
+        channel = 'B';
+    }
+ 
+    memset(data, 0, sizeof(data));
+    datalen = ais_binary_encode(&session->gpsdata.ais, &data[0], 0);
+    if (datalen > 6*60) {
+	static int number1 = 0;
+        msg1 = datalen / (6*60);
+	if ((datalen % (6*60)) != 0) {
+	    msg1 += 1;
+	}
+	/*@ +charint */
+	numc[0] = '0' + (char)(number1 & 0x0f);
+	/*@ -charint */
+	numc[1] = '\0';
+	number1 += 1;
+	if (number1 > 9) {
+	    number1 = 0;
+	}
+	offset = 0;
+	for (msg2=1;msg2<=msg1;msg2++) {
+	    unsigned char old;
+
+	    old = '\0';
+	    if (strlen((char *)&data[(msg2-1)*60]) > 60) {
+	        old = data[(msg2-0)*60];
+	        data[(msg2-0)*60] = '\0';
+	    }
+	    if (datalen >= (6*60)) {
+	        left = 0;
+		datalen -= 6*60;
+	    } else {
+	        left = GETLEFT(datalen);
+	    }
+	    (void)snprintf(&bufp[offset], len-offset,
+			   "%s,%u,%u,%s,%c,%s,%u",
+			   type,
+			   msg1,
+			   msg2,
+			   numc,
+			   channel,
+			   (char *)&data[(msg2-1)*60],
+			   left);
+
+	    nmea_add_checksum(&bufp[offset]);
+	    if (old != (unsigned char)'\0') {
+		data[(msg2-0)*60] = old;
+	    }
+	    offset = (unsigned int) strlen(bufp);
+	}
+    } else {
+        msg1 = 1;
+	msg2 = 1;
+	numc[0] = '\0';
+        left = GETLEFT(datalen);
+	(void)snprintf(bufp, len,
+		       "%s,%u,%u,%s,%c,%s,%u",
+		       type,
+		       msg1,
+		       msg2,
+		       numc,
+		       channel,
+		       (char *)data,
+		       left);
+
+	nmea_add_checksum(bufp);
+    }
+
+    if (session->gpsdata.ais.type == 24) {
+        msg1 = 1;
+	msg2 = 1;
+	numc[0] = '\0';
+
+        memset(data, 0, sizeof(data));
+	datalen = ais_binary_encode(&session->gpsdata.ais, &data[0], 1);
+	left = GETLEFT(datalen);
+	offset = (unsigned int)strlen(bufp);
+	(void)snprintf(&bufp[offset], len-offset,
+		       "%s,%u,%u,%s,%c,%s,%u",
+		       type,
+		       msg1,
+		       msg2,
+		       numc,
+		       channel,
+		       (char *)data,
+		       left);
+	nmea_add_checksum(bufp+offset);
+    }
+}
+#endif /* AIVDM_ENABLE */
+
 /*@-compdef -mustdefine@*/
 /* *INDENT-OFF* */
 void nmea_tpv_dump(struct gps_device_t *session,
@@ -316,6 +425,17 @@ void nmea_subframe_dump(struct gps_device_t *session,
 				   len - strlen(bufp));
 }
 
+#ifdef AIVDM_ENABLE
+void nmea_ais_dump(struct gps_device_t *session,
+		   /*@out@*/ char bufp[], size_t len)
+{
+    bufp[0] = '\0';
+    if ((session->gpsdata.set & AIS_SET) != 0)
+	gpsd_binary_ais_dump(session, bufp + strlen(bufp),
+				   len - strlen(bufp));
+}
+#endif /* AIVDM_ENABLE */
+
 /*@+compdef +mustdefine@*/
 
 /* pseudonmea.c ends here */
