@@ -96,6 +96,8 @@ int json_watch_read(const char *buf,
 	                                  .nodefault = true},
 	{"scaled",         t_boolean,  .addr.boolean = &ccp->scaled},
 	{"timing",         t_boolean,  .addr.boolean = &ccp->timing},
+	{"split24",        t_boolean,  .addr.boolean = &ccp->split24},
+	{"pps",            t_boolean,  .addr.boolean = &ccp->pps},
 	{"device",         t_string,   .addr.string = ccp->devpath,
 	                                  .len = sizeof(ccp->devpath)},
 	{"remote",         t_string,   .addr.string = ccp->remote,
