@@ -2,15 +2,41 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
-#include <stdio.h>
-#ifndef S_SPLINT_S
-#include <unistd.h>
-#endif /* S_SPLINT_S */
-#include <stdlib.h>
 #include <string.h>
 #include "gpsd_config.h"
 
+/*
+ * These versions use memcpy and strlen() because they are often
+ * heavily optimized down to assembler level. Thus, likely to be
+ * faster even with the function call overhead.
+ */
+
 #ifndef HAVE_STRLCAT
+/*
+ * Appends src to string dst of size siz (unlike strncat, siz is the
+ * full size of dst, not space left).  At most siz-1 characters
+ * will be copied.  Always NUL terminates (unless siz <= strlen(dst)).
+ * Returns strlen(src) + MIN(siz, strlen(initial dst)).
+ * If retval >= siz, truncation occurred.
+ */
+/*@ -compdef -mayaliasunique -mustdefine @*/
+size_t strlcat(char *dst, const char *src, size_t siz)
+{
+    size_t slen = strlen(src);
+    size_t dlen = strlen(dst);
+    if (siz != 0) {
+	if (dlen + slen < siz)
+	    memcpy(dst + dlen, src, slen + 1);
+	else {
+	    memcpy(dst + dlen, src, siz - dlen - 1);
+	    dst[siz - 1] = '\0';
+	}
+    }
+    return dlen + slen;
+}
+/*@ +compdef +mayaliasunique +mustdefine @*/
+
+#ifdef __UNUSED__
 /*	$OpenBSD: strlcat.c,v 1.13 2005/08/08 08:05:37 espie Exp $	*/
 
 /*
@@ -29,13 +55,6 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/*
- * Appends src to string dst of size siz (unlike strncat, siz is the
- * full size of dst, not space left).  At most siz-1 characters
- * will be copied.  Always NUL terminates (unless siz <= strlen(dst)).
- * Returns strlen(src) + MIN(siz, strlen(initial dst)).
- * If retval >= siz, truncation occurred.
- */
 /*@ -usedef -mustdefine @*/
 size_t strlcat(char *dst, const char *src, size_t siz)
 {
@@ -63,11 +82,32 @@ size_t strlcat(char *dst, const char *src, size_t siz)
 
     return (dlen + (s - src));	/* count does not include NUL */
 }
-
+#endif /* __UNUSED__ */
 /*@ +usedef +mustdefine @*/
 #endif /* HAVE_STRLCAT */
 
 #ifndef HAVE_STRLCPY
+/*
+ * Copy src to string dst of size siz.  At most siz-1 characters
+ * will be copied.  Always NUL terminates (unless siz == 0).
+ * Returns strlen(src); if retval >= siz, truncation occurred.
+ */
+/*@ -mayaliasunique -mustdefine @*/
+size_t strlcpy(char *dst, const char *src, size_t siz)
+{
+    size_t len = strlen(src);
+    if (siz != 0) {
+	if (len >= siz) {
+	    memcpy(dst, src, siz - 1);
+	    dst[siz - 1] = '\0';
+	} else
+	    memcpy(dst, src, len + 1);
+    }
+    return len;
+}
+/*@ -mayaliasunique -mustdefine @*/
+
+#ifdef __UNUSED__
 /*	$OpenBSD: strlcpy.c,v 1.11 2006/05/05 15:27:38 millert Exp $	*/
 
 /*
@@ -85,12 +125,6 @@ size_t strlcat(char *dst, const char *src, size_t siz)
  * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-
-/*
- * Copy src to string dst of size siz.  At most siz-1 characters
- * will be copied.  Always NUL terminates (unless siz == 0).
- * Returns strlen(src); if retval >= siz, truncation occurred.
- */
 size_t strlcpy(char *dst, const char *src, size_t siz)
 {
     char *d = dst;
@@ -115,4 +149,5 @@ size_t strlcpy(char *dst, const char *src, size_t siz)
 
     return ((size_t) (s - src - 1));	/* count does not include NUL */
 }
+#endif /* __UNUSED__ */
 #endif /* HAVE_STRLCPY */
