@@ -20,7 +20,7 @@ static short sw1, sw2;
 static unsigned short uw1, uw2;
 static int sl1, sl2;
 static unsigned int ul1, ul2;
-static long long sL1, sL2;
+static int64_t sL1, sL2;
 static uint64_t uL1, uL2;
 static float f1;
 static double d1;
@@ -135,54 +135,25 @@ int main(int argc, char *argv[])
 	 7, 2, 2, false, "2 bits crossing 1st to 2nd byte (0x1923)"},
     };
 
-    unsigned char *sp;
-
     memcpy(buf, "\x01\x02\x03\x04\x05\x06\x07\x08", 8);
     memcpy(buf + 8, "\xff\xfe\xfd\xfc\xfb\xfa\xf9\xf8", 8);
     memcpy(buf + 16, "\x40\x09\x21\xfb\x54\x44\x2d\x18", 8);
     memcpy(buf + 24, "\x40\x49\x0f\xdb", 4);
     /*@ +observertrans +usereleased @*/
 
+    if (!quiet)
+	(void)printf("Testing bitfield extraction\n");
+
     /*@-type@*/
     sb1 = getsb(buf, 0);
     sb2 = getsb(buf, 8);
     ub1 = getub(buf, 0);
     ub2 = getub(buf, 8);
-    sw1 = getbes16(buf, 0);
-    sw2 = getbes16(buf, 8);
-    uw1 = getbeu16(buf, 0);
-    uw2 = getbeu16(buf, 8);
-    sl1 = getbes32(buf, 0);
-    sl2 = getbes32(buf, 8);
-    ul1 = getbeu32(buf, 0);
-    ul2 = getbeu32(buf, 8);
-    sL1 = getbes64(buf, 0);
-    sL2 = getbes64(buf, 8);
-    uL1 = getbeu64(buf, 0);
-    uL2 = getbeu64(buf, 8);
-    f1 = getbef32((const char *)buf, 24);
-    d1 = getbed64((const char *)buf, 16);
-    sb1 = getsb(buf, 0);
-    sb2 = getsb(buf, 8);
-    ub1 = getub(buf, 0);
-    ub2 = getub(buf, 8);
-    sw1 = getles16(buf, 0);
-    sw2 = getles16(buf, 8);
-    uw1 = getleu16(buf, 0);
-    uw2 = getleu16(buf, 8);
-    sl1 = getles32(buf, 0);
-    sl2 = getles32(buf, 8);
-    ul1 = getleu32(buf, 0);
-    ul2 = getleu32(buf, 8);
-    sL1 = getles64(buf, 0);
-    sL2 = getles64(buf, 8);
-    uL1 = getleu64(buf, 0);
-    uL2 = getleu64(buf, 8);
-    f1 = getlef32((const char *)buf, 24);
-    d1 = getled64((const char *)buf, 16);
     /*@+type@*/
 
     if (!quiet) {
+	unsigned char *sp;
+
 	(void)fputs("Test data:", stdout);
 	for (sp = buf; sp < buf + 28; sp++)
 	    (void)printf(" %02x", *sp);
@@ -190,10 +161,42 @@ int main(int argc, char *argv[])
 
 	/* big-endian test */
 	printf("Big-endian:\n");
+	/*@-type@*/
+	sw1 = getbes16(buf, 0);
+	sw2 = getbes16(buf, 8);
+	uw1 = getbeu16(buf, 0);
+	uw2 = getbeu16(buf, 8);
+	sl1 = getbes32(buf, 0);
+	sl2 = getbes32(buf, 8);
+	ul1 = getbeu32(buf, 0);
+	ul2 = getbeu32(buf, 8);
+	sL1 = getbes64(buf, 0);
+	sL2 = getbes64(buf, 8);
+	uL1 = getbeu64(buf, 0);
+	uL2 = getbeu64(buf, 8);
+	f1 = getbef32((const char *)buf, 24);
+	d1 = getbed64((const char *)buf, 16);
+	/*@+type@*/
 	bedumpall();
 
 	/* little-endian test */
 	printf("Little-endian:\n");
+	/*@-type@*/
+	sw1 = getles16(buf, 0);
+	sw2 = getles16(buf, 8);
+	uw1 = getleu16(buf, 0);
+	uw2 = getleu16(buf, 8);
+	sl1 = getles32(buf, 0);
+	sl2 = getles32(buf, 8);
+	ul1 = getleu32(buf, 0);
+	ul2 = getleu32(buf, 8);
+	sL1 = getles64(buf, 0);
+	sL2 = getles64(buf, 8);
+	uL1 = getleu64(buf, 0);
+	uL2 = getleu64(buf, 8);
+	f1 = getlef32((const char *)buf, 24);
+	d1 = getled64((const char *)buf, 16);
+	/*@+type@*/
 	ledumpall();
     }
 
@@ -202,7 +205,6 @@ int main(int argc, char *argv[])
     if (ub1 != 1)  printf("getub(buf, 0) FAILED\n");
     if (ub2 != 0xff) printf("getub(buf, 8) FAILED\n");
 
-    (void)printf("Testing bitfield extraction\n");
     for (up = unsigned_tests;
 	 up <
 	 unsigned_tests + sizeof(unsigned_tests) / sizeof(unsigned_tests[0]);
